<?php

//  ***************************************************************************
//
//  ndfdBrowserClientByDay.php - Client for viewing NDFD XML in a browser  
//
//  John L. Schattel                    MDL                        13 July 2007
//  Red Hat Linux                                                 Apache Server
//
//  PURPOSE:  This code serves to call the NDFD XML SOAP server and display the
//            resulting DWML document in the browser.
//
//  VARIABLES:
//
//     xmlOut - This is the string that holds the NDFD XML (DWML) output
//     zipCode - User supplied zip code
//     latitude - User supplied latitude (South latitude is negative)
//     longitude - User supplied longitude (West Longitude is negative)
//     latLons - An array containing all the latitude and longitude
//               pairs (i.e. 30.00,-77.00 40.00,-111.00 39.7,104.8)
//               that are to be processed. The array also holds the 
//               start and end times and the sector for each lat/lon
//               pair.
//     startDate - User supplied local time they want data for.  This
//                 input is entered as a XML string (ex. 2004-04-13)
//     numDays - The integer number of days the user wants data returned for
//     format - Defines the length of each period.  The length can be either
//              24-hourly or 12-hourly.
//     callingType - The type of call being made.  If it is browser, then the
//                   DWML is returned to the browser.  If it is a SOAP request
//                   the DWML is returned to the SOAP server. 
//
//  CALLED ROUTINES:
//
//     generateDWMLByDay() - This program retrieves the NDFD data requested by
//                           the user and then formats it as NDFD DWMLgenByDay.
//     getFormatElementList()  - Formats the list of NDFD elements that are used
//                               in the call to degrib.  This indicates which
//                               elements the user wants formatted in the output
//                               DWML document
//     
//  SOURCE CODE CONTROL INFORMATION
//
//      Name:
//         %PM%
//         %PID%
//  
//      Status:
//         %PS%
//  
//      History:
//         %PL%
//  
//      Change Document History:
//         %PIRC%
//
//  ***************************************************************************

//  Turn on logging
$startLogging = FALSE;

//  See what time it is to assist in measuring server performance
$startServerTime = array_sum(explode(' ',microtime()));

//  ==================  FUNCTIONS  ============================================

require('../../DWMLgen/weatherParameters.inc');
//  This routine formats the list of NDFD elements needed by degrib
require_once('../../lib/CMD_Options/getFormatElementList.inc');

//  This routine converts the user supplied local time to the UTC time required 
//  by degrib 
require_once('../../lib/time/convertToUTC_Time.inc');

//  This routine creates and returns to the user the NDFDgenByDay product
require(dirname(__FILE__) . "/../../DWMLgenByDay/generateDWMLByDay.inc");

//  Bring in the logging code
require_once('../../SOAP_server/logRequest.inc');

//  ==================  MAIN PROGRAM  =========================================

// Send the appropriate mime type for XML
header("Content-Type: text/xml");

//  Tell backend code that its being invoked by the browser client
$callingType = "browser";

//sanitize the form inputs
$startDate = strip_tags($_GET['startDate']);
$numDays = strip_tags($_GET['numDays']);
$format = strip_tags($_GET['format']);
$Unit = strip_tags($_GET['Unit']);

//  *****************************************************************************
//  Processing a SINGLE POINT call for an NDFDgenByDay product (24- and 12 hourly)
//  *****************************************************************************
if (array_key_exists('lat',$_GET) && ($_GET['lat'] != "")) {
	$latitude = 0;
	$longitude = 0;
	//check if the lat/lon values are really a valid number
	if(is_numeric($_GET['lat'])){
		$latitude = strip_tags($_GET['lat']);
	}
	if(is_numeric($_GET['lon'])){
		$longitude = strip_tags($_GET['lon']);
	}
	
   	//  Initialize the user supplied input variables
   	//$latitude  = $lat;
   	//$longitude = $lon;

   	//  Create the latitude and longitude array expected by generateDWMLByDay.inc
   	$latLons[0]['lat'] = $latitude;
   	$latLons[0]['lon'] = $longitude;

    //  Execute the code to create and return to the user the NDFDgenByDay product
    generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Log information if required
   	if ($startLogging){
    	logRequest("DWMLgenByDay Single Point",$format,$startServerTime,1,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
	}
}

//  *****************************************************************************
//  Processing a MULTI POINT call for an NDFDgenByDay product (24- and 12 hourly)
//  *****************************************************************************
elseif (array_key_exists('listLatLon',$_GET) && ($_GET['listLatLon'] != "")){
   	//  This routine determines the degrib -XMLFormatOnly command line input
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  Initialize the user supplied input variables
   	$listLatLon = strip_tags($_GET['listLatLon']);

   	//  Parse the list of latitude and longitude pairs into an array
   	$latLons = parseLatLonList($listLatLon);

   	//  Execute the code to create and return to the user the NDFDgenByDay product
   	generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);
      	logRequest("DWMLgenByDay Multiple Points",$format,$startServerTime,$numPoints,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a SUBGRID call for an NDFDgenByDay product (24- and 12 hourly)
//  *****************************************************************************
elseif (array_key_exists('lat1',$_GET) && ($_GET['lat1'] != "")){
	//  This routine determines a default resolution needed for the subgrid
   	require_once('../../lib/conversion/getDefaultResolution.inc');

   	//  This routine parses a list of latitude and longitude values into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  This routine creates a list of latitude and longitude pairs contained in
   	//  a rectangular subgrid.
   	require_once('../../subgrid/generateLatLonList.inc');

   	//  This routine lists the four corner latitude and longitude pairs for an
   	//  NDFD grid.
   	require_once('../../subgrid/generateCornerLatLonPairs.inc');

   	//  This routine to extract the contents of the <latLonList> element
   	require_once('../../lib/conversion/parseDWMLelement.inc');

   	//  This not a situation where the user provided us a resolution, so we will
   	//  need the sector that their points are on
   	$resolutionSub = strip_tags($_GET['resolutionSub']);
   	
	//  Convert subgrid corners to component latitude and longitude values
   	if(validateNumericData($_GET['lat1'])){
		$lowerLeftLat  = strip_tags($_GET['lat1']);
	}else{
		$lowerLeftLat = 0;
	}

	if(validateNumericData($_GET['lat2'])){
		$upperRightLat  = strip_tags($_GET['lat2']);
	}else{
		$upperRightLat = 0;
	}

	if(validateNumericData($_GET['lon1'])){
		$lowerLeftLon  = strip_tags($_GET['lon1']);
	}else{
		$lowerLeftLon = 0;
	}

	if(validateNumericData($_GET['lon2'])){
		$upperRightLon  = strip_tags($_GET['lon2']);
	}else{
		$upperRightLon = 0;
	}
   	
   	if ((! isset($resolutionSub)) || ($resolutionSub == "") || ($resolutionSub == "default")){
  		//  Initialize the weatherParameters array
      	require_once('../../SOAP_server/selectSector.php');

      	//  Determine if the user's points are on any NDFD grids
      	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $lowerLeftLat,$lowerLeftLon", $sector1, $returnValue);

      	//  Accommodate RTMA conus 2.5km res.
      	$numSectorsForPoint = count($sector1);
      	for ($k = 0; $k < $numSectorsForPoint; $k++){
         	if (($sector1[$k] == "conus5") || ($sector1[$k] == "conus2_5")){
            	$sector1[$k] = "conus";
      		}
      	}
      	//  Determine if the user's points are on any NDFD grids
      	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $upperRightLat,$upperRightLon", $sector2, $returnValue);

      	//  Accommodate RTMA conus 2.5km res.
      	$numSectorsForPoint = count($sector2);
      	for ($k = 0; $k < $numSectorsForPoint; $k++){
         	if (($sector2[$k] == "conus5") || ($sector2[$k] == "conus2_5")){
            	$sector2[$k] = "conus";
      		}
   		}
      	//  Get the matching sector
      	$sectorsDoNotMatch = selectSector($sector1,$sector2,$sector,$sector1List,$sector2List);

      	// check to ensure that both points are on the same NDFD sector
      	if ($sectorsDoNotMatch){
         	exit("<error><h2>ERROR</h2><pre>Subgrid corners must be in same sector.  Point $lowerLeftLat,$lowerLeftLon in \"$sector1List\" and point $upperRightLat,$upperRightLon in \"$sector2List\"</pre></error>\n");
      	}
   	}//end of if ((! isSet($_GET['resolutionSub'])) || ($_GET['resolutionSub'] == "") || ($_GET['resolutionSub'] == "default"))

   	//  Now lets deal with user who didn't give us any resolution input
   	if ((! isset($resolutionSub)) || ($resolutionSub == "")){
      	$resolutionSub = getDefaultResolution($sector);
   	}

   	//  Now we handle the rare case where the user wants the whole grid
   	//  Of course they are only going to get the max allowed grid points
   	else if ($resolutionSub == "default"){
      	//  Tell generateCornerLatLonPairs.inc that its being invoked by the SOAP server
      	$callingType = "soap";

      	//  Get the resolution
      	$LatLonList = generateCornerLatLonPairs($sector,$callingType);
      	$LatLonListwResolution = $LatLonList;

      	//  Extract the latitude and longitude list
      	$LatLonList = parseDWMLelement($LatLonList,"<latLonList>","</latLonList>");

      	//  We want the zeroth (lower left) and 2nd (upper right) array elements
      	$newCorners = explode(" ",$LatLonList);

      	$lowerLeftPoint = explode(",",$newCorners[0]);
      	$lowerLeftLat  = $lowerLeftPoint[0];
      	$lowerLeftLon  = $lowerLeftPoint[1];

      	$upperRightPoint = explode(",",$newCorners[0]);
      	$upperRightLat = $upperRightPoint[0];
      	$upperRightLon = $upperRightPoint[1];

      	//  Extract the resolution
      	$resolutionSub = parseDWMLelement($LatLonListwResolution,"<minResolution>","</minResolution>");
   	}

   	//  Initialize the user supplied input variables
   	$resolution    = $resolutionSub;

   	//  Tell generateLatLonLis.inc that its being invoked by a SOAP client
   	//  so we get the data into the variable not standard out
   	$callingType = "soap";

   	//  get the list of latitude and longitude pairs
   	$LatLonList = generateLatLonList($lowerLeftLat,$lowerLeftLon,$upperRightLat,$upperRightLon,$resolution,$callingType);

   	//  Extract the latitude and longitude list
   	$LatLonList = parseDWMLelement($LatLonList,"<latLonList>","</latLonList>");

   	//  Parse the list of latitude and longitude pairs into an array
   	$latLons = parseLatLonList($LatLonList);

   	//  Tell generateDWMLgenByDay.inc that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create and return to the user the NDFDgenByDay product
   	generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);
      	logRequest("DWMLgenByDay Corners Subgrid",$format,$startServerTime,$numPoints,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
   	}
}
//  *****************************************************************************
//  Processing a LINE OF POINTS call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('endPoint1Lat',$_GET) && ($_GET['endPoint1Lat'] != "")){
   	//  This routine creates the array of latitudes and longitudes used to create
   	//  the degrib -pnt command line input
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  Get code to deterine if corner points are on matching sectors
   	require_once('../../SOAP_server/selectSector.php');

	if(validateNumericData($_GET['endPoint1Lat'])){
   		$endPoint1Lat = strip_tags($_GET['endPoint1Lat']);	
   	}else{
   		$endPoint1Lat = 0;
   	}
   	
   	if(validateNumericData($_GET['endPoint1Lon'])){
   		$endPoint1Lon = strip_tags($_GET['endPoint1Lon']);
   	}else{
   		$endPoint1Lon = 0;
   	}
   	
   	if(validateNumericData($_GET['endPoint2Lat'])){
   		$endPoint2Lat = strip_tags($_GET['endPoint2Lat']);	
   	}else{
   		$endPoint2Lat = 0;
   	}
   	
   	if(validateNumericData($_GET['endPoint2Lon'])){
   		$endPoint2Lon = strip_tags($_GET['endPoint2Lon']);
   	}else{
   		$endPoint2Lon = 0;
   	}

   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $endPoint1Lat,$endPoint1Lon", $sector1, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector1);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector1[$k] == "conus5") || ($sector1[$k] == "conus2_5")){
         	$sector1[$k] = "conus";
   		}
   	}
   	
   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $endPoint2Lat,$endPoint2Lon", $sector2, $returnValue);

    //  Accommodate RTMA conus 2.5km res.
    $numSectorsForPoint = count($sector2);
    for ($k = 0; $k < $numSectorsForPoint; $k++){
    	if (($sector2[$k] == "conus5") || ($sector2[$k] == "conus2_5")){
            $sector2[$k] = "conus";
      	}
    }
    
   	//  Get the matching sector
   	$sectorsDoNotMatch = selectSector($sector1,$sector2,$sector,$sector1List,$sector2List);

   	// check to ensure that both points are on the same NDFD sector
   	if ($sectorsDoNotMatch){
      	exit("<error><h2>ERROR</h2><pre>End points must be in same sector.  Point $endPoint1Lat,$endPoint1Lon in \"$sector1List\" and point $endPoint2Lat,$endPoint2Lon in \"$sector2List\"</pre></error>\n");
   	}
   
   	//  Parse listLatLon into an array
   	exec("../../bin/formatLatLonForLine $sector $endPoint1Lat $endPoint1Lon $endPoint2Lat $endPoint2Lon", $listOfPoints, $returnValue);

   	//  Parse listLatLon into an array
   	$latLons = parseLatLonList($listOfPoints[0]);

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);
      	logRequest("DWMLgenByDay Line",$format,$startServerTime,$numPoints,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
   	}
}
//  *****************************************************************************
//  Processing a ZIP CODE call for an NDFDgenByDay product (24- and 12 hourly)
//  *****************************************************************************
elseif (array_key_exists('zipCodeList',$_GET) && ($_GET['zipCodeList'] != "")){
   	//  This routine parses a list of lat/lons into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	// Routine to Convert zip code to lat/lon
   	require_once('../../SOAP_server/zipToLatLon.php');

   	//  Initialize the user supplied input variables
   	$zipCodeList = strip_tags($_GET['zipCodeList']);

   	// Convert zip code to lat/lon
   	$dataLatLon = zipToLatLon($zipCodeList, $data='', $message);

   	//  If error messages were returned, print them out
   	if (preg_match("/Error/",$dataLatLon, $matches)){
      	exit("<errorMessage>$dataLatLon</errorMessage>\n");
   	}

   	//  Parse listLatLon into an array
   	$latLons = parseLatLonList($dataLatLon);

   	//  Tell zipToLatLon that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Log information if required
	if ($startLogging){
      	$numPoints = count($latLons);
      	logRequest("DWMLgenByDay Zip Codes",$format,$startServerTime,$numPoints,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a CITIES call for an NDFDgenByDay product (24- and 12 hourly)
//  *****************************************************************************
elseif (array_key_exists('citiesLevel',$_GET) && ($_GET['citiesLevel'] != "")){
   	//  This routine parses a list of lat/lons into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  This code supplies the arrays or cities list names and lat/lons
   	require_once('../../lib/conversion/cityLatLonValues.inc');

   	//  This code replaces <point> elements with a <city> elements
   	require_once('../../lib/conversion/insertCityNames.inc');

   	//  Initialize the user supplied input variable
   	if(validateNumericData($_GET['citiesLevel'])){
   		$level = strip_tags($_GET['citiesLevel']);
   	}else{
   		$level = 0;
   	}
   	
   	//  Parse cityData lat and lon listlist in citylatLonValues.inc into an array
   	$latLons = parseLatLonList($cityData[$level]['latLonList']);

   	//  Tell generateDWMLByDay that its being invoked by the SOAP client so it will return 
   	//  the DWML and we can capture it in a variable
   	$callingType = "soap";

   	//  Execute the code to create and return to the user the DWMLgen product
   	$DWMLdoc = generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Get the city names for these latitude longitude pairs
   	$cityNames = $cityData[$level]['nameList'];

   	//  Insert the city names into the DWML product
   	$DWMLdoc = insertCityNames($DWMLdoc,$cityNames);

   	//  Output the updated DWML document
   	echo $DWMLdoc;

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);
      	logRequest("DWMLgenByDay Cities $level",$format,$startServerTime,$numPoints,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
   	}
}
//  *****************************************************************************
//  Processing a SQUARE (central point and direction) call for an NDFDgenByDay product (24- and 12 hourly)
//  *****************************************************************************
elseif (array_key_exists('centerPointLat',$_GET) && ($_GET['centerPointLat'] != "")){
   	//  This routine determines a default resolution needed for the subgrid
   	require_once('../../lib/conversion/getDefaultResolution.inc');

   	//  This routine parses a list of latitude and longitude values into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  This routine creates a list of latitude and longitude pairs contained in
   	//  a rectangular subgrid.
   	require_once('../../subgrid/generateLatLonList.inc');

	if(validateNumericData($_GET['centerPointLat'])){
   		$centerPointLat = strip_tags($_GET['centerPointLat']);
   	}else{
   		$centerPointLat = 0;
   	}
   		
   	if(validateNumericData($_GET['centerPointLon'])){
   		$centerPointLon = strip_tags($_GET['centerPointLon']);
   	}else{
   		$centerPointLon = 0;
   	}
   	
   	if(validateNumericData($_GET['distanceLat'])){
   		$distanceLat = strip_tags($_GET['distanceLat']);
   	}else{
   		$distanceLat = 0;
   	}
   	
   	if(validateNumericData($_GET['distanceLon'])){
   		$distanceLon = strip_tags($_GET['distanceLon']);
   	}else{
   		$distanceLon = 0;
   	}

   	//  Get the sector that contains the center point
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $centerPointLat,$centerPointLon", $sector, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
    $numSectorsForPoint = count($sector);
    for ($k = 0; $k < $numSectorsForPoint; $k++){
    	if (($sector[$k] == "conus5") || ($sector[$k] == "conus2_5")){
            $sector[$k] = "conus";
      	}
    }
    
   	// check to ensure that point is on an NDFD grid
   	if (empty($sector)){
      	exit("<error><h2>ERROR</h2><pre>Center point \"$centerPointLat,$centerPointLon\" is not on an NDFD grid.</pre></error>\n");
   	}
   	
   	//  If the user did not provide a resolution use the default
   	$resolutionSquare = strip_tags($_GET['resolutionSquare']);
   	if (! isset($resolutionSquare) || ($resolutionSquare == "")){
      	$resolution = getDefaultResolution($sector[0]);
   	}else{
      	$resolution = $resolutionSquare;	
   	}
   	
   	//  Get the list of latitude and longitude pairs contained in the rectangle
   	exec("../../bin/formatLatLonForSquare {$sector[0]} $centerPointLat $centerPointLon $distanceLat $distanceLon $resolution", $listOfPoints, $returnValue);

   	//  Parse the list of latitude and longitude pairs into an array
   	$latLons = parseLatLonList($listOfPoints[0]);

   	//  Tell generateDWMLgenByDay.inc that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWMLByDay($latLons,$startDate,$numDays,$Unit,$format,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);
      	logRequest("DWMLgenByDay Center Point Subgrid",$format,$startServerTime,$numPoints,5,"maxt,mint,pop12,wx,icons","nonSOAP","logWEB_SERV1_1");
   	}
}
//  *****************************************************************************
//  We didn't find input that we were expecting, so tell the user of the error
//  *****************************************************************************
else{
   	echo "<error><h2>Bad service input(s)</h2><pre>";
  	print_r($_GET);
   	echo "</pre></error>\n";
}

?>
