<?php

//  ***************************************************************************
//
//  ndfdXMLclient.php - Client for viewing NDFD XML in a browser 
//
//  John L. Schattel                    MDL                        13 July 2007
//  Red Hat Linux                                                 Apache Server
//
//  PURPOSE:  This code serves to call the NDFD XML SOAP server and display the
//            resulting DWML document in the browser.
//
//  VARIABLES:
//
//     xmlOut - This is the string that holds the NDFD XML (DWML) output
//     zipCodeList - User supplied zip code
//     latitude - User supplied latitude (South latitude is negative)
//     longitude - User supplied longitude (West Longitude is negative)
//     latLons - An array containing all the latitude and longitude
//               pairs (i.e. 30.00,-77.00 40.00,-111.00 39.7,104.8)
//               that are to be processed. The array also holds the 
//               start and end times and the sector for each lat/lon
//               pair.
//     product - User supplied product (time-series, glance, etc.)
//     startTime - User supplied local time they want data for.  This
//                         input is entered as a XML string 
//                         (ex. 2004-04-13T12:00:00)
//     endTime - User supplied local time they want data for.  This
//                         input is entered as a XML string 
//                         (ex. 2004-04-13T12:00:00)
//     weatherParameters - Associative array of strings to indicate
//                         which NDFD parameters they want returned.
//                         Possible values include the following:
//
//     weatherParameters['maxt']  = TRUE returns maximum temperature
//     weatherParameters['mint']  = TRUE returns minimum temperature
//     weatherParameters['temp']  = TRUE returns 3 hourly temperature
//     weatherParameters['dew']   = TRUE returns dewpoint temperature
//     weatherParameters['pop12'] = TRUE returns POP12
//     weatherParameters['qpf']   = TRUE returns QPF
//     weatherParameters['sky']   = TRUE returns sky cover
//     weatherParameters['snow']  = TRUE returns snow amount
//     weatherParameters['wspd']  = TRUE returns wind speed
//     weatherParameters['wdir']  = TRUE returns wind direction
//     weatherParameters['wx']    = TRUE returns weather
//     weatherParameters['waveh'] = TRUE returns wave height
//     weatherParameters['icons'] = TRUE returns weather icon links
//     weatherParameters['rh']    = TRUE returns relative humidity
//     weatherParameters['appt']  = TRUE returns apparent temperature
//     weatherParameters['incw34']  = TRUE returns Probability of a 
//                                    Tropical Cyclone Wind Speed >34 
//                                    Knots (Incremental)
//     weatherParameters['incw50']  = TRUE returns Probability of a 
//                                    Tropical Cyclone Wind Speed >50 
//                                    Knots (Incremental)
//     weatherParameters['incw64']  = TRUE returns Probability of a 
//                                    Tropical Cyclone Wind Speed >64 
//                                    Knots (Incremental)
//     weatherParameters['cumw34']  = TRUE returns Probability of a 
//                                    Tropical Cyclone Wind Speed >34 
//                                    Knots (Cumulative)
//     weatherParameters['cumw50']  = TRUE returns Probability of a 
//                                    Tropical Cyclone Wind Speed >50 
//                                    Knots (Cumulative)
//     weatherParameters['cumw64']  = TRUE returns Probability of a 
//                                    Tropical Cyclone Wind Speed >64 
//                                    Knots (Cumulative)
//     weatherParameters['critfireo']     = TRUE returns Fire Wx risk 
//                                          from wind and relative humidity
//     weatherParameters['dryfireo']      = TRUE returns Fire Wx risk
//                                          from dry thunderstorms.
//     weatherParameters['conhazo']       = TRUE returns probability of
//                                          convection.
//     weatherParameters['ptornado']      = TRUE returns probability of
//                                          tornadoes.
//     weatherParameters['phail']         = TRUE returns probability of
//                                          hail.
//     weatherParameters['ptstmwinds']    = TRUE returns probability of
//                                          damaging thunderstorms.
//     weatherParameters['pxtornado']     = TRUE returns probability of
//                                          extreme tornadoes.
//     weatherParameters['pxhail']        = TRUE returns probability of
//                                          extreme hail.
//     weatherParameters['pxtstmwinds']   = TRUE returns probability of
//                                          extreme thunderstorm winds.
//     weatherParameters['ptotsvrtstm']   = TRUE returns probability of
//                                          severe thunderstorms.
//     weatherParameters['pxtotsvrtstm']  = TRUE returns probability of
//                                          extreme severe thunderstorms.
//     weatherParameters['wgust']      = TRUE returns wind gust
//
//     callingType - Flag to indicate that we want to have the DWML displayed in the browser
//
//  CALLED ROUTINES:
//
//     generateDWML() - This program retrieves the NDFD data requested by
//                      the user and then formats it as NDFD XML.
//     convertToUTC_Time()    - Converts a user supplied local time to UTC.
//     getFormatElementList() - Formats the list of NDFD elements that are used
//                              in the call to degrib.  This indicates which
//                              elements the user wants formatted in the output
//                              DWML document
//     
//  SOURCE CODE CONTROL INFORMATION
//
//      Name:
//         %PM%
//         %PID%
//  
//      Status:
//         %PS%
//  
//      History:
//         %PL%
//  
//      Change Document History:
//         %PIRC%
//
//  ***************************************************************************
//  Turn on logging
$startLogging = FALSE;

//  See what time it is to assist in measuring server performance
$startServerTime = array_sum(explode(' ',microtime()));

//  ==================  FUNCTIONS  ============================================

//  This routine formats the list of NDFD elements needed by degrib
require_once('../../lib/CMD_Options/getFormatElementList.inc');

//  This routine converts the user supplied local time to the UTC time required 
//  by degrib 
require_once('../../lib/time/convertToUTC_Time.inc');

//  This routine creates and returns to the user the DWMLgen product
//require_once('../../DWMLgen/generateDWML.inc');
require(dirname(__FILE__) . "/../../DWMLgen/generateDWML.inc");

//  Bring in the logging code
//require_once('../../SOAP_server/logRequest.inc');

//  ==================  MAIN PROGRAM  =========================================
// Send the appropriate mime type for XML
header("Content-Type: text/xml");

//  Tell backend code that its being invoked by the browser client
$callingType = "browser";

//  Initialize the weatherParameters array
require('../../DWMLgen/weatherParameters.inc');
$productArray = array("time-series", "glance");
$date_pattern = "/^[0-9]{4}\-[0-9]{2}\-[0-9]{2}[T]{1}[0-9]{2}\:[0-9]{2}\:[0-9]{2}/";
//sanitize the URL parameters

if(validateNumericData($_GET['lat'])){
	$latitude = strip_tags($_GET['lat']);
}else{
	$latitude = 0;
}
   	
if(validateNumericData($_GET['lon'])){
	$longitude = strip_tags($_GET['lon']);
}else{
	$longitude = 0;
}
   	
if(in_array($_GET['product'], $productArray)){
	$product = strip_tags($_GET['product']);	
}else{
	$product = "time-series";
}
   	
if(preg_match($date_pattern, $_GET['begin'], $begin_found)){
	$startTime = strip_tags($_GET['begin']);	
}else{
	$startTime = "";
}
   	
if(preg_match($date_pattern, $_GET['end'], $end_found)){
	$endTime = strip_tags($_GET['end']);	
}else{
	$endTime = "";
}

if( ($_GET['Unit'] == 'e') || ($_GET['Unit'] == 'm') ){
	$Unit = strip_tags($_GET['Unit']);
}else{
	$Unit = "e";
}

//  *****************************************************************************
//  Processing a SINGLE POINT call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
if (array_key_exists('lat',$_GET) && ($_GET['lat'] != "")) {
	
   	//  Create the latitude and longitude array expected by generateDWML.inc
   	$latLons[0]['lat'] = $latitude;
   	$latLons[0]['lon'] = $longitude;

    //  Execute the code to create and return to the user the DWMLgen product
   	generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Log information if required
   	if ($startLogging){
    	if ($product == "time-series"){
      		$numElements = 0;
       		$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      
      	logRequest("DWMLgen Single Point",$product,$startServerTime,1,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a MULTI POINT call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('listLatLon',$_GET) && ($_GET['listLatLon'] != "")){
	//  This routine determines the degrib -XMLFormatOnly command line input
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  Initialize the user supplied input variables
   	
	$listLatLon = strip_tags($_GET['listLatLon']); 
   	//  Parse the list of latitude and longitude pairs into an array
   	$latLons = parseLatLonList($listLatLon);

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Log information if required
   	if ($startLogging){
  		$numPoints = count($latLons);

      	if ($product == "time-series"){
         	$numElements = 0;
         	$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      	logRequest("DWMLgen Multiple Points",$product,$startServerTime,$numPoints,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a SUBGRID call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('lat1',$_GET) && ($_GET['lat1'] != "")){
	//  This routine determines a default resolution needed for the subgrid
   	require_once('../../lib/conversion/getDefaultResolution.inc');

   	//  This routine parses a list of latitude and longitude values into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  This routine lists the four corner latitude and longitude pairs for an
   	//  NDFD grid.
   	require_once('../../subgrid/generateCornerLatLonPairs.inc');

   	//  This routine to extract the contents of the <latLonList> element
   	require_once('../../lib/conversion/parseDWMLelement.inc');

   	//  This routine creates a list of latitude and longitude pairs contained in
   	//  a rectangular subgrid.
   	require_once('../../subgrid/generateLatLonList.inc');

   	$resolutionSub = strip_tags($_GET['resolutionSub']);
   	
	//  Convert subgrid corners to component latitude and longitude values
   	if(validateNumericData($_GET['lat1'])){
		$lowerLeftLat  = strip_tags($_GET['lat1']);
	}else{
		$lowerLeftLat = 0;
	}

	if(validateNumericData($_GET['lat2'])){
		$upperRightLat  = strip_tags($_GET['lat2']);
	}else{
		$upperRightLat = 0;
	}

	if(validateNumericData($_GET['lon1'])){
		$lowerLeftLon  = strip_tags($_GET['lon1']);
	}else{
		$lowerLeftLon = 0;
	}

	if(validateNumericData($_GET['lon2'])){
		$upperRightLon  = strip_tags($_GET['lon2']);
	}else{
		$upperRightLon = 0;
	}
   	
   	//  This not a situation where the user provided us a resolution, so we will
   	//  need the sector that their points are on
   	if ((!isset($resolutionSub)) || ($resolutionSub == "") || ($resolutionSub == "default")){
   		
    	//  Get code to deterine if corner points are on matching sectors
      	require_once('../../SOAP_server/selectSector.php');

      	//  Determine if the user's points are on any NDFD grids
      	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $lowerLeftLat,$lowerLeftLon", $sector1, $returnValue);

      	//  Accommodate RTMA conus 2.5km res.
      	$numSectorsForPoint = count($sector1);
      	for ($k = 0; $k < $numSectorsForPoint; $k++){
        	if (($sector1[$k] == "conus5") || ($sector1[$k] == "conus2_5")){
            	$sector1[$k] = "conus";
        	}
      	}

      	//  Determine if the user's points are on any NDFD grids
      	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $upperRightLat,$upperRightLon", $sector2, $returnValue);

      	//  Accommodate RTMA conus 2.5km res.
      	$numSectorsForPoint = count($sector2);
      	for ($k = 0; $k < $numSectorsForPoint; $k++){
         	if (($sector2[$k] == "conus5") || ($sector2[$k] == "conus2_5")){
            	$sector2[$k] = "conus";
      		}
      	}

      	//  Get the matching sector
      	$sectorsDoNotMatch = selectSector($sector1,$sector2,$sector,$sector1List,$sector2List);

      	// check to ensure that both points are on the same NDFD sector
      	if ($sectorsDoNotMatch){
         	exit("<error><h2>ERROR</h2><pre>Subgrid corners must be in same sector.  Point $lowerLeftLat,$lowerLeftLon in \"$sector1List\" and point $upperRightLat,$upperRightLon in \"$sector2List\"</pre></error>\n");
   		}
   	}
   
   	//  Now lets deal with user who didn't give us any resolution input
   	if ((! isset($resolutionSub)) || ($resolutionSub == "")){
      	$resolutionSub = getDefaultResolution($sector);
   	}

   	//  Now we handle the rare case where the user wants the whole grid
   	//  Of course they are only going to get the max allowed grid points
   	else if ($resolutionSub == "default"){
      	//  Tell generateCornerLatLonPairs.inc that its being invoked by the SOAP server
      	$callingType = "soap";

      	//  Get the resolution
      	$LatLonList = generateCornerLatLonPairs($sector,$callingType);
      	$LatLonListwResolution = $LatLonList;

      	//  Extract the latitude and longitude list
      	$LatLonList = parseDWMLelement($LatLonList,"<latLonList>","</latLonList>");

      	//  We want the zeroth (lower left) and 2nd (upper right) array elements
      	$newCorners = explode(" ",$LatLonList);

      	$lowerLeftPoint = explode(",",$newCorners[0]);
      	//$_GET['lat1']  = $lowerLeftPoint[0];
      	$lat1  = $lowerLeftPoint[0];
      	//$_GET['lon1']  = $lowerLeftPoint[1];
		$lon1  = $lowerLeftPoint[1];
      	
      	$upperRightPoint = explode(",",$newCorners[0]);
      	//$_GET['lat2'] = $upperRightPoint[0];
      	$lat2 = $upperRightPoint[0];
      	//$_GET['lon2'] = $upperRightPoint[1];
      	$lon2 = $upperRightPoint[1];

      	//  Extract the resolution
      	$resolutionSub = parseDWMLelement($LatLonListwResolution,"<minResolution>","</minResolution>");
   	}

   	$resolution    = $_GET['resolutionSub'];

   	//  Tell generateLatLonLis.inc that its being invoked by a SOAP client
   	//  so we get the data into the variable not standard out
   	$callingType = "soap";

   	//  get the list of latitude and longitude pairs
   	$LatLonList = generateLatLonList($lowerLeftLat,$lowerLeftLon,$upperRightLat,$upperRightLon,$resolution,$callingType);

   	//  Extract the latitude and longitude list
   	$LatLonList = parseDWMLelement($LatLonList,"<latLonList>","</latLonList>");

   	//  Parse the list of latitude and longitude pairs into an array
   	$latLons = parseLatLonList($LatLonList);

   	//  Tell generateDWMLgen.inc that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Log information if required
   	if ($startLogging){
    	$numPoints = count($latLons);

      	if ($product == "time-series"){
        	$numElements = 0;
         	$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      
      	logRequest("DWMLgen Corners Subgrid",$product,$startServerTime,$numPoints,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a request for a LIST LAT LON for NDFD points in a SUBGRID
//  *****************************************************************************
elseif (array_key_exists('listLat1',$_GET) && ($_GET['listLat1'] != "")){
   	//  Initialize the global number of points variable set in generateLatLonList.inc
   	$numPoints = -1;

   	//  This routine determines the default resolution given a specific sector
   	require_once('../../lib/conversion/getDefaultResolution.inc');

   	//  This routine creates a list of latitude and longitude pairs contained in
   	//  a rectangular subgrid.
   	require_once('../../subgrid/generateLatLonList.inc');

   	//  Get code to deterine if corner points are on matching sectors
   	require_once('../../SOAP_server/selectSector.php');

   	//  Convert subgrid corners to component latitude and longitude values
   	$listLat1 = strip_tags($_GET['listLat1']);
	$listLon1 = strip_tags($_GET['listLon1']);
	$listLat2 = strip_tags($_GET['listLat2']);
	$listLon2 = strip_tags($_GET['listLon2']);
   	
   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $listLat1,$listLon1", $sector1, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector1);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector1[$k] == "conus5") || ($sector1[$k] == "conus2_5")){
         	$sector1[$k] = "conus";
   		}
   	}

   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $listLat2,$listLon2", $sector2, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector2);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector2[$k] == "conus5") || ($sector2[$k] == "conus2_5")){
         	$sector2[$k] = "conus";
   		}
   	}
   
   	//  Get the matching sector
   	$sectorsDoNotMatch = selectSector($sector1,$sector2,$sector,$sector1List,$sector2List);

   	// check to ensure that both points are on the same NDFD sector
   	if ($sectorsDoNotMatch){
      	exit("<error><h2>ERROR</h2><pre>Subgrid corners must be in same sector.  Point $listLat1,$listLon1 in \"$sector1List\" and point $listLat2,$listLon2 in \"$sector2List\"</pre></error>\n");
   	}
   	
   	//  Some times the client will be called without a resolution so check for it
   	if ((array_key_exists('resolutionList',$_GET)) && ($_GET['resolutionList'] != "")){
      	$resolution = $_GET['resolutionList'];
   	}
   	else{
      	$resolution = getDefaultResolution($sector);
      	if ("$resolution" == "Bad Sector"){
         	exit("<error><h2>ERROR</h2><pre>Bad Sector \"$sector\"</pre></error>\n");
   		}
   	}
   
   	//  Tell generateLatLonList.inc that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create list of lat/lon pairs in the user request subgrid
   	generateLatLonList($listLat1,$listLon1,$listLat2,$listLon2,$resolution,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	logRequest("LatLonList Corners Subgrid",$resolution,$startServerTime,$numPoints,0,"none","nonSOAP","logWEB_SERV1_1");
	}
}

//  *****************************************************************************
//  Processing a LINE OF POINTS call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('endPoint1Lat',$_GET) && ($_GET['endPoint1Lat'] != "")){
	//  This routine creates the array of latitudes and longitudes used to create
   	//  the degrib -pnt command line input
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  Get code to deterine if line end points are in matching sectors
   	require_once('../../SOAP_server/selectSector.php');

   	if(validateNumericData($_GET['endPoint1Lat'])){
   		$endPoint1Lat = strip_tags($_GET['endPoint1Lat']);	
   	}else{
   		$endPoint1Lat = 0;
   	}
   	
   	if(validateNumericData($_GET['endPoint1Lon'])){
   		$endPoint1Lon = strip_tags($_GET['endPoint1Lon']);
   	}else{
   		$endPoint1Lon = 0;
   	}
   	
   	if(validateNumericData($_GET['endPoint2Lat'])){
   		$endPoint2Lat = strip_tags($_GET['endPoint2Lat']);	
   	}else{
   		$endPoint2Lat = 0;
   	}
   	
   	if(validateNumericData($_GET['endPoint2Lon'])){
   		$endPoint2Lon = strip_tags($_GET['endPoint2Lon']);
   	}else{
   		$endPoint2Lon = 0;
   	}
   	//$product      = $_GET['product'];
   	//$startTime    = $_GET['begin'];
   	//$endTime      = $_GET['end'];
   	//$Unit         = $_GET['Unit'];

   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $endPoint1Lat,$endPoint1Lon", $sector1, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector1);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector1[$k] == "conus5") || ($sector1[$k] == "conus2_5")){
         	$sector1[$k] = "conus";
   		}
   	}
   
   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $endPoint2Lat,$endPoint2Lon", $sector2, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector2);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector2[$k] == "conus5") || ($sector2[$k] == "conus2_5")){
         	$sector2[$k] = "conus";
   		}
   	}
   	
   	//  Get the matching sector
   	$sectorsDoNotMatch = selectSector($sector1,$sector2,$sector,$sector1List,$sector2List);

   	// check to ensure that both points are on the same NDFD sector
   	if ($sectorsDoNotMatch){
      	exit("<error><h2>ERROR</h2><pre>End points must be in same NDFD sector.  Point $endPoint1Lat,$endPoint1Lon in \"$sector1List\" and point $endPoint2Lat,$endPoint2Lon in \"$sector2List\"</pre></error>\n");
   	}
   	
   	//  Parse listLatLon into an array
   	exec("../../bin/formatLatLonForLine $sector $endPoint1Lat $endPoint1Lon $endPoint2Lat $endPoint2Lon", $listOfPoints, $returnValue);

   	//  Parse listLatLon into an array
   	$latLons = parseLatLonList($listOfPoints[0]);

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);

      	if ($product == "time-series"){
         	$numElements = 0;
         	$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      	
      	logRequest("DWMLgen Line",$product,$startServerTime,$numPoints,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a LIST LAT LON call for LINE OF POINTS
//  *****************************************************************************
elseif (array_key_exists('listEndPoint1Lat',$_GET) && ($_GET['listEndPoint1Lat'] != "")){
   	//  Get function to determine if end points are on matching sector
   	require_once('../../SOAP_server/selectSector.php');

   	$listEndPoint1Lat = strip_tags($_GET['listEndPoint1Lat']);
   	$listEndPoint1Lon = strip_tags($_GET['listEndPoint1Lon']);
   	$listEndPoint2Lat = strip_tags($_GET['listEndPoint2Lat']);
   	$listEndPoint2Lon = strip_tags($_GET['listEndPoint2Lon']);

   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $listEndPoint1Lat,$listEndPoint1Lon", $sector1, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector1);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
    	if (($sector1[$k] == "conus5") || ($sector1[$k] == "conus2_5")){
         	$sector1[$k] = "conus";
   		}
   	}
   	
   	//  Determine if the user's points are on any NDFD grids
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $listEndPoint2Lat,$listEndPoint2Lon", $sector2, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector2);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector2[$k] == "conus5") || ($sector2[$k] == "conus2_5")){
         	$sector2[$k] = "conus";
   		}
   	}
   	
   	//  Get the matching sector
   	$sectorsDoNotMatch = selectSector($sector1,$sector2,$sector,$sector1List,$sector2List);

   	// check to ensure that both points are on the same NDFD sector
   	if ($sectorsDoNotMatch){
     	exit("<error><h2>ERROR</h2><pre>End points must be in same NDFD sector. Point $listEndPoint1Lat,$listEndPoint1Lon in \"$sector1List\" and point $listEndPoint2Lat,$listEndPoint2Lon in \"$sector2List\"</pre></error>\n");
   	}
   
   	//  Parse listLatLon into an array
   	exec("../../bin/formatLatLonForLine $sector $listEndPoint1Lat $listEndPoint1Lon $listEndPoint2Lat $listEndPoint2Lon", $listOfPoints, $returnValue);

   	//  Wrap the list of latitude and longitude in DWML document
   	require_once('../../lib/conversion/formatLatLonList.inc');
   	$outputList = formatLatLonList($listOfPoints[0]);

   	//  Output the list of lat lons
   	echo $outputList;

   	//  Log information if required
   	if ($startLogging){
      	$lineArray = explode(" ",$listOfPoints[0]);
      	$numPoints = count($lineArray);

      	logRequest("LatLonList Line",$sector,$startServerTime,$numPoints,0,"none","nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a ZIP CODE call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('zipCodeList',$_GET) && ($_GET['zipCodeList'] != "")){
   	//  This routine parses a list of lat/lons into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	// Routine to Convert zip code to lat/lon
   	require_once('../../SOAP_server/zipToLatLon.php');

   	//  Initialize the user supplied input variables
   	$zipCodeList = strip_tags($_GET['zipCodeList']);
  
   	// Convert zip code to lat/lon
   	$dataLatLon = zipToLatLon($zipCodeList, $data='', $message);

   	//  If error messages were returned, print them out
   	if (preg_match("/Error/",$dataLatLon, $matches)){
      	exit("<errorMessage>$dataLatLon</errorMessage>\n");
   	}

   	//  Parse listLatLon into an array
   	$latLons = parseLatLonList($dataLatLon);

   	//  Tell zipToLatLon that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);

      	if ($product == "time-series"){
         	$numElements = 0;
         	$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         	
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      
      	logRequest("DWMLgen Zip Codes",$product,$startServerTime,$numPoints,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a LIST LAT LON for ZIP CODES call
//  *****************************************************************************
elseif (array_key_exists('listZipCodeList',$_GET) && ($_GET['listZipCodeList'] != "")){
   	//  Wrap the list of latitude and longitude in DWML document
   	require_once('../../lib/conversion/formatLatLonList.inc');

   	// Routine to Convert zip code to lat/lon
   	require_once('../../SOAP_server/zipToLatLon.php');

   	//  Initialize the user supplied input variables
   	$listZipCodeList = strip_tags($_GET['listZipCodeList']);

   	// Convert zip code to lat/lon
   	$dataLatLon = zipToLatLon($listZipCodeList, $data='', $message);

   	//  If error messages were returned, print them out
   	if (preg_match("/Error/",$dataLatLon, $matches)){
      	exit("<errorMessage>$dataLatLon</errorMessage>\n");
   	}

   	//  Wrap the list of latitude and longitude in DWML document
   	$outputList = formatLatLonList($dataLatLon);

   	//  Output the list of lat lons
   	echo $outputList;

   	//  Log information if required
   	if ($startLogging){
      	$zipCodeArray = explode(" ",$listZipCodeList);
      	$numPoints = count($zipCodeArray);

      	logRequest("LatLonList Zip Codes","NA",$startServerTime,$numPoints,0,"none","nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a Cities call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('citiesLevel',$_GET) && ($_GET['citiesLevel'] != "")){
   	//  This routine parses a list of lat/lons into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	//  This code supplies the arrays or cities list names and lat/lons
   	require_once('../../lib/conversion/cityLatLonValues.inc');

   	//  This code replaces <point> elements with a <city> elements
   	require_once('../../lib/conversion/insertCityNames.inc');

   	//  Initialize the user supplied input variables
	if(validateNumericData($_GET['citiesLevel'])){
   		$level = strip_tags($_GET['citiesLevel']);
   	}else{
   		$level = 0;
   	}
   	//  Parse cityData lat and lon listlist in citylatLonValues.inc into an array
   	$latLons = parseLatLonList($cityData[$level]['latLonList']);

   	//  Tell generateDWML that its being invoked by the SOAP client so it will return 
   	//  the DWML and we can capture it in a variable
   	$callingType = "soap";

   	//  Execute the code to create and return to the user the DWMLgen product
   	$DWMLdoc = generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Get the city names for these latitude longitude pairs
   	$cityNames = $cityData[$level]['nameList'];

   	//  Insert the city names into the DWML product
   	$DWMLdoc = insertCityNames($DWMLdoc,$cityNames);

   	//  Output the updated DWML document
   	echo $DWMLdoc;

   	//  If error messages were returned, print them out
   	if ($errorMessage != ""){
     	exit("<errorMessage>$errorMessage</errorMessage>\n");
   	}
   
   	//  Log information if required
   	if ($startLogging){
      	if ($level == 1){
         	$numPoints = 78;
      	}else if ($level == 2){
         	$numPoints = 115;
      	}else if ($level == 3){
         	$numPoints = 200;
      	}else if ($level == 4){
         	$numPoints = 103;
      	}else if ($level == 12){
         	$numPoints = 193;
      	}else if ($level == 34){
         	$numPoints = 303;
      	}else if ($level == 1234){
         	$numPoints = 496;
      	}
      	
      	if ($product == "time-series"){
         	$numElements = 0;
         	$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      	
      	logRequest("DWMLgen Cities $level",$product,$startServerTime,$numPoints,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a LIST LAT LON for Cities call
//  *****************************************************************************
elseif (array_key_exists('listCitiesLevel',$_GET) && ($_GET['listCitiesLevel'] != "")){
   	//  Wrap the list of latitude and longitude in DWML document
   	require_once('../../lib/conversion/formatCityList.inc');

   	//  Wrap the list of latitude and longitude in DWML document
   	require_once('../../lib/conversion/cityLatLonValues.inc');

   	//  Initialize the user supplied input variables
   	$level = strip_tags($_GET['listCitiesLevel']);

   	//  Wrap the list of latitude and longitude in DWML document
   	$outputList = formatCityList($cityData[$level]['latLonList'],$cityData[$level]['nameList']);

   	//  Output the list of lat lons
   	echo $outputList;

   	//  Log information if required
   	if ($startLogging){
      	if ($level == 1){
         	$numPoints = 78;
      	}else if ($level == 2){
         	$numPoints = 115;
      	}else if ($level == 3){
         	$numPoints = 200;
      	}else if ($level == 4){
         	$numPoints = 103;
      	}else if ($level == 12){
         	$numPoints = 193;
      	}else if ($level == 34){
         	$numPoints = 303;
      	}else if ($level == 1234){
         	$numPoints = 496;
      	}
      	
      	logRequest("LatLonList Cities $level",$level,$startServerTime,$numPoints,0,"none","nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a SQUARE (central point and direction) call for an NDFDgen product (time-series or glance)
//  *****************************************************************************
elseif (array_key_exists('centerPointLat',$_GET) && ($_GET['centerPointLat'] != "")){
   	//  This routine determines a default resolution needed for the subgrid
   	require_once('../../lib/conversion/getDefaultResolution.inc');

   	//  This routine parses a list of latitude and longitude values into an array
   	require_once('../../lib/CMD_Options/parseLatLonList.inc');

   	if(validateNumericData($_GET['centerPointLat'])){
   		$centerPointLat = strip_tags($_GET['centerPointLat']);
   	}else{
   		$centerPointLat = 0;
   	}
   		
   	if(validateNumericData($_GET['centerPointLon'])){
   		$centerPointLon = strip_tags($_GET['centerPointLon']);
   	}else{
   		$centerPointLon = 0;
   	}
   	
   	if(validateNumericData($_GET['distanceLat'])){
   		$distanceLat = strip_tags($_GET['distanceLat']);
   	}else{
   		$distanceLat = 0;
   	}
   	
   	if(validateNumericData($_GET['distanceLon'])){
   		$distanceLon = strip_tags($_GET['distanceLon']);
   	}else{
   		$distanceLon = 0;
   	}
   	//  Get the sector that contains the center point
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $centerPointLat,$centerPointLon", $sector, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector[$k] == "conus5") || ($sector[$k] == "conus2_5")){
         	$sector[$k] = "conus";
   		}
   	}
   
   	// check to ensure that point is on an NDFD grid
   	if (empty($sector)){
      	exit("<error><h2>ERROR</h2><pre>Center point \"$centerPointLat,$centerPointLon\" is not on an NDFD grid.</pre></error>\n");
   	}
   
   	//  If the user did not provide a resolution use the default
   	$resolutionSquare = strip_tags($_GET['resolutionSquare']);
   	if (! isset($resolutionSquare) || ($resolutionSquare == "")){
      	$resolution = getDefaultResolution($sector[0]);
   	}else{
      	$resolution = $resolutionSquare;
   	}
   
   	//  Get the list of latitude and longitude pairs contained in the rectangle
   	exec("../../bin/formatLatLonForSquare {$sector[0]} $centerPointLat $centerPointLon $distanceLat $distanceLon $resolution", $listOfPoints, $returnValue);

   	//  Parse the list of latitude and longitude pairs into an array
   	$latLons = parseLatLonList($listOfPoints[0]);

   	//  Tell generateDWMLgen.inc that its being invoked by the browser client
   	$callingType = "browser";

   	//  Execute the code to create and return to the user the DWMLgen product
   	generateDWML($latLons,$product,$startTime,$endTime,$Unit,$weatherParameters,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	$numPoints = count($latLons);

      	if ($product == "time-series"){
         	$numElements = 0;
         	$elements = "";
         	$indexNames = array_keys($weatherParameters);
         	$numIndexNames = count($indexNames);
         
         	for ($i = 0; $i < $numIndexNames; $i++){
            	if ($weatherParameters[$indexNames[$i]]){
               		$numElements++;
               		$elements .= $indexNames[$i] . ",";
            	}
         	}
         
         	//  Remove the last training comma that we added
         	$elements = rtrim($elements,",");
      	}
      	else if ($product == "glance"){
         	$numElements = 5;
         	$elements = "maxt,mint,sky,wx,icons,wwa";
      	}
      	else{
         	$numElements = -1;
      	}
      	
      	logRequest("DWMLgen Center Point Subgrid",$product,$startServerTime,$numPoints,$numElements,$elements,"nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a LIST LAT LON for SQUARE (central point and direction) call
//  *****************************************************************************
elseif (array_key_exists('listCenterPointLat',$_GET) && ($_GET['listCenterPointLat'] != "")){
   	//  This routine determines a default resolution needed for the subgrid
   	require_once('../../lib/conversion/getDefaultResolution.inc');

   	//  This routine encodes the list of latitude and longitude in DWML document
   	require_once('../../lib/conversion/formatLatLonList.inc');

   	$listCenterPointLat = strip_tags($_GET['listCenterPointLat']);
   	$listCenterPointLon = strip_tags($_GET['listCenterPointLon']);
   	$listDistanceLat = strip_tags($_GET['listDistanceLat']);
   	$listDistanceLon = strip_tags($_GET['listDistanceLon']);

   	//  Get the sector that contains the center point
   	exec("/ndfdv2/local/ndfd/bin/degrib_DP.cur -Sector -pnt $listCenterPointLat,$listCenterPointLon", $sector, $returnValue);

   	//  Accommodate RTMA conus 2.5km res.
   	$numSectorsForPoint = count($sector);
   	for ($k = 0; $k < $numSectorsForPoint; $k++){
      	if (($sector[$k] == "conus5") || ($sector[$k] == "conus2_5")){
         	$sector[$k] = "conus";
   		}
   	}
   	
   	// check to ensure that point is on an NDFD grid
   	if (empty($sector)){
      	exit("<error><h2>ERROR</h2><pre>Center point \"$listCenterPointLat,$listCenterPointLon\" is not on an NDFD grid.</pre></error>\n");
   	}
   	
   	$listResolutionSquare = strip_tags($_GET['listResolutionSquare']);
   	//  If the user did not provide a resolution use the default
   	if (! isset($listResolutionSquare) || ($listResolutionSquare == "")){
      	$resolution = getDefaultResolution($sector[0]);
   	}else{
      	$resolution = $listResolutionSquare;
   	}
   	
   	//  Get the list of latitude and longitude pairs contained in the rectangle
   	exec("../../bin/formatLatLonForSquare {$sector[0]} $listCenterPointLat $listCenterPointLon $listDistanceLat $listDistanceLon $resolution", $listOfPoints, $returnValue);

   	//  Wrap the list of latitude and longitude in DWML document
   	$outputList = formatLatLonList($listOfPoints[0],-999.9);

   	//  Output the list of lat lons
   	echo $outputList;

   	//  Log information if required
   	if ($startLogging){
      	$squareArray = explode(" ",$listOfPoints[0]);
      	$numPoints = count($squareArray);

      	logRequest("LatLonList Center Point Subgrid",$sector[0],$startServerTime,$numPoints,0,"none","nonSOAP","logWEB_SERV1_1");
   	}
}

//  *****************************************************************************
//  Processing a request for a list of lat/lons for NDFD points in a subgrid
//  *****************************************************************************
elseif (array_key_exists('sector',$_GET) && ($_GET['sector'] != "")){
   	//  This routine lists the four corner latitude and longitude pairs for an NDFD grid.
   	require_once('../../subgrid/generateCornerLatLonPairs.inc');

   	//  Tell generateCornerLatLonPairs that its being invoked by the browser client
   	$callingType = "browser";

   	//  Get the resolution
   	$sector = strip_tags($_GET['sector']);
   	generateCornerLatLonPairs($sector,$callingType);

   	//  Log information if required
   	if ($startLogging){
      	logRequest("LatLonList Four Corners",$sector,$startServerTime,4,0,"none","nonSOAP","logWEB_SERV1_1");
	}
}

//  *****************************************************************************
//  Processing a GML call for NDFD data (single time or a period of time)
//  *****************************************************************************
elseif (array_key_exists('gmlListLatLon',$_GET) && ($_GET['gmlListLatLon'] != "")){
	//require_once('../../lib/CMD_Options/getFormatElementList.inc');
	
   	//  Initialize the user supplied input variables
   	$_GET['LATLONLIST'] = strip_tags($_GET['gmlListLatLon']);
  	$_GET['SERVICE']    = "WS";
   	$_GET['TYPENAME']   = strip_tags($_GET['featureType']);
   	$_GET['REQUEST']    = "GetFeature";

   	//  Some requests provide a single time input, its there save it.
   	//  Otherwise the $_GET variable should have a startTime, endTime, and
   	//  compType array elements, which is which need no additional processing here.
   	if (array_key_exists('requestedTime',$_GET) && ($_GET['requestedTime'] != "")){
   		if(preg_match($date_pattern, $_GET['requestedTime'], $requested_found)){
			$requestedTime = strip_tags($_GET['requestedTime']);
		}else{
			$requestedTime = "None";
		}
      	$_GET['TIME'] = $requestedTime;
   	}
   	
   	//  If the NDFD elements are provided in the propertyName input, us it
   	if (array_key_exists('propertyName',$_GET) && ($_GET['propertyName'] != "")){
      	$_GET['ELEMENTS'] = strip_tags($_GET['propertyName']);
   	}
   	//  Otherwise use the weatherParameters input
   	else{
      	$_GET['ELEMENTS'] = getFormatElementList($weatherParameters);
   	}
   
   	//  Be prepared to process multiple feature types
   	$featureTypeArray = explode(",",$_GET['featureType']);
   
   	//  Loop over each feature until we find a valid one, then stop
   	for ($i = 0; $i < count($featureTypeArray); $i++){
      	switch ($featureTypeArray[$i]){
         	case "Forecast_Gml2Point"     :
         	case "Forecast_Gml2AllWx"     :
            	$_GET['VERSION'] = "1.0.0";
            	break;

         	case "Forecast_GmlsfPoint"    :
         	case "Forecast_GmlObs"        :
         	case "NdfdMultiPointCoverage" :
         	case "Ndfd_KmlPoint"          :
            	$_GET['VERSION'] = "1.1.0";
            	break;
      	}
   	}
	//print_r($_GET); exit;
   	//  This routine determines the degrib -XMLFormatOnly command line input
   	include('../../OGC_services/ndfdOWSserver.php');
}

//  *****************************************************************************
//  We didn't find input that we were expecting, so tell the user of the error
//  *****************************************************************************
else{
   echo "<error><h2>Bad service input(s)</h2><pre>";
   print_r($_GET);
   echo "</pre></error>\n";
}

?>
